import {
  REGEXP_TYPE,
  SYMBOL_TYPE,
  RULE_CONDITION_TYPE,
  CALCULATE_TYPE,
  OPERATOR_FUN_TYPE,
  GROUP_TYPE
} from '@/utils/crfForm/constants'
import formulaRule from '@/utils/crfForm/formulaRule'
/**
 *  正则表达式类型
 */
export default {
  /**
   * 获取常用正则配置
   */
  getRegexpData: function (id) {
    const regex = this.REGEXP_DATAS.filter(d => d.id.toString() === id.toString())
    return regex ? regex[0] : null
  },
  /**
   * 获取自定义规则配置
   */
  getCustomData: function (id) {
    const regex = this.CUSTOM_RULES.filter(d => d.id.toString() === id.toString())
    return regex ? regex[0] : null
  },
  /**
   * element rule 自定义函数实例
   */
  checkAge: (rule, value, callback) => {
    if (!value) {
      return callback(new Error('年龄不能为空'))
    }
    if (!Number.isInteger(value)) {
      callback(new Error('请输入数字值'))
    } else {
      if (value < 18) {
        callback(new Error('必须年满18岁'))
      } else {
        callback()
      }
    }
  },
  /**
   * 数字范围验证
   */
  numberRange: function (min, max) {
    return (rule, value, callback, source, options) => {
      if (!value) {
        return callback()
      }
      if (value < min && value < max) {
        callback(new Error('请输入大于等于' + min + '的数字'))
      } else if (value > min && value > max) {
        callback(new Error('请输入小于等于' + max + '的数字'))
      } else if (value < min && !max) {
        // 最大数没填，取值范围为：[min, +∞)
        callback(new Error('请输入大于等于' + min + '的数字'))
      } else if (value > max && !min) {
        // 最小数没填，取值范围为：(-∞, max]
        callback(new Error('请输入小于等于' + max + '的数字'))
      } else {
        callback()
      }
    }
  },
  /**
   * 小数规则，如果decimal=false，在输入小数时直接去掉，不提示
   */
  decimalRule: function (decimal, decimalLength) {
    return (rule, value, callback) => {
      if (!value) {
        return callback()
      }
      if (!decimal) {
        // 是整数
        value = Number(value)
        Number.isInteger(value) ? callback() : callback(new Error('请输入整数'))
      } else if (decimal) {
        // 是小数
        if (value.toString().indexOf('.') !== -1) {
          if (value.toString().split('.')[1].length > decimalLength) {
            callback(new Error('请输入保留' + decimalLength + '位的小数'))
          } else {
            callback()
          }
        } else {
          callback()
        }
      }
    }
  },
  /**
   * 符号验证规则，依赖常量：SYMBOL_TYPE
   */
  symbolRule: function (plusMinus) {
    return (rule, value, callback) => {
      if (!value) {
        return callback()
      }
      switch (plusMinus) {
        // 正负
        case SYMBOL_TYPE.symbol:
          callback()
          break
        case SYMBOL_TYPE.plus:
          if (Math.sign(value) === 1) {
            callback()
          } else {
            callback(new Error('请输入正数'))
          }
          break
        case SYMBOL_TYPE.minus:
          if (Math.sign(value) === -1) {
            callback()
          } else {
            callback(new Error('请输入负数'))
          }
          break
      }
    }
  },
  /**
   * 中国身份证验证
   */
  chinaIdentityValid: (rule, value, callback) => {
    if (!value) {
      return callback()
    }
    let city = {
      11: '北京',
      12: '天津',
      13: '河北',
      14: '山西',
      15: '内蒙古',
      21: '辽宁',
      22: '吉林',
      23: '黑龙江 ',
      31: '上海',
      32: '江苏',
      33: '浙江',
      34: '安徽',
      35: '福建',
      36: '江西',
      37: '山东',
      41: '河南',
      42: '湖北 ',
      43: '湖南',
      44: '广东',
      45: '广西',
      46: '海南',
      50: '重庆',
      51: '四川',
      52: '贵州',
      53: '云南',
      54: '西藏 ',
      61: '陕西',
      62: '甘肃',
      63: '青海',
      64: '宁夏',
      65: '新疆',
      71: '台湾',
      81: '香港',
      82: '澳门',
      91: '国外'
    }
    if (!value || !/^\d{6}(18|19|20)?\d{2}(0[1-9]|1[012])(0[1-9]|[12]\d|3[01])\d{3}(\d|X)$/i.test(value)) {
      callback(new Error('身份证号格式错误'))
    } else if (!city[value.substr(0, 2)]) {
      callback(new Error('身份证号格式错误'))
    } else {
      // 18位身份证需要验证最后一位校验位
      if (value.length === 18) {
        const code = value.split('')
        // ∑(ai×Wi)(mod 11)
        // 加权因子
        const factor = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        // 校验位
        const parity = [1, 0, 'X', 9, 8, 7, 6, 5, 4, 3, 2]
        let sum = 0
        let ai = 0
        let wi = 0
        for (let i = 0; i < 17; i++) {
          ai = code[i]
          wi = factor[i]
          sum += ai * wi
        }
        if (parity[sum % 11].toString() !== code[17]) {
          callback(new Error())
        }
      }
    }
    callback()
  },
  REGEXP_DATAS: [{
    id: REGEXP_TYPE.phone,
    name: '手机号',
    /* eslint-disable-next-line */
    regexp: '^1(3|4|5|7|8)\\d{9}$',
    jsFunciton: '',
    message: '请输入正确的手机号'
  }, {
    id: REGEXP_TYPE.telephone,
    name: '电话号码',
    /* eslint-disable-next-line */
    regexp: '^(\\(\\d{3,4}\\)|\\d{3,4}-|\\s)?\\d{7,14}$',
    jsFunciton: null,
    message: '请输入正确的电话号码'
  }, {
    // 使用element验证实现
    id: REGEXP_TYPE.email,
    type: 'email',
    name: '邮箱',
    regexp: null,
    jsFunciton: null,
    message: '请输入正确的Email格式'
  }, {
    // 使用element验证实现
    id: REGEXP_TYPE.url,
    type: 'url',
    name: 'URL',
    regexp: null,
    jsFunciton: null,
    message: '请输入正确的URL格式'
  }, {
    id: REGEXP_TYPE.letterNum,
    name: '字母+数字',
    /* eslint-disable-next-line */
    regexp: '^\\w+$',
    jsFunciton: null,
    message: '请输入字母+数字的格式'
  }, {
    id: REGEXP_TYPE.chinaCard,
    name: '中国身份证',
    regexp: null,
    jsFunciton: 'chinaIdentityValid',
    message: '身份证格式错误'
  }, {
    id: REGEXP_TYPE.custom,
    name: '自定义正则',
    regexp: null,
    jsFunciton: null,
    message: '格式错误'
  }],
  /**
   * 预设规则
   */
  CUSTOM_RULES: [{
    id: 1,
    name: '计算依据规则',
    jsFunciton: 'calculationBasisValid',
    message: '专项经费、自筹经费都为0，计算依据必须填 ‘无’ 或 ‘/’'
  }],
  /**
   * 计算依据规则实现
   */
  calculationBasisValid: (rule, value, callback) => {
    let hasSpecial = true
    let hasSelfPlan = true
    for (let field of rule.line.fieldList) {
      if (field.name === '专项经费' && field.defValue === '0') {
        hasSpecial = false
      } else if (field.name === '自筹经费' && field.defValue === '0') {
        hasSelfPlan = false
      }
    }
    if (!hasSpecial && !hasSelfPlan && value !== '0' && value !== '/') {
      callback(new Error())
    }
    callback()
  },
  getFieldDefvalue: function (field, superGroupField) {
    let result
    for (let d of superGroupField) {
      if (field.lineNum !== '' && field.lineNum !== null) {
        // 如果存在threeArr[0].lineNum,因为0也算存在所以+1
        if (d.id === field.fieldId && d.lineNum === (field.lineNum + 1)) {
          result = d.defValue
        }
      } else {
        if (d.id === field.fieldId) {
          result = d.defValue
        }
      }
    }
    return result
  },
  getOperatorSwitch: function (d) {
    let operator
    switch (d.operatorType) {
      // 第二个必为符号
      case OPERATOR_FUN_TYPE.add:
        operator = '+'
        break
      case OPERATOR_FUN_TYPE.minus:
        operator = '-'
        break
      case OPERATOR_FUN_TYPE.multiply:
        operator = '*'
        break
      case OPERATOR_FUN_TYPE.division:
        operator = '/'
        break
    }
    return operator
  },
  checkRelationalOperator: function (operator) {
    // 返回公式中equal:5, greaterThan: 6,lessThan: 7,greaterThanAndEqual: 8,lessThanAndEqual: 9,notEqual: 10,的index
    return (operator.operatorType === OPERATOR_FUN_TYPE.equal || operator.operatorType === OPERATOR_FUN_TYPE.greaterThan || operator.operatorType === OPERATOR_FUN_TYPE.lessThan || operator.operatorType === OPERATOR_FUN_TYPE.greaterThanAndEqual || operator.operatorType === OPERATOR_FUN_TYPE.lessThanAndEqual || operator.operatorType === OPERATOR_FUN_TYPE.notEqual)
  },
  /**
   * 自定义规则
  */
  customRule: function (conditionList, resultList) {
    return (rule, value, callback) => {
      let superGroupField = []
      for (let item of rule.superGroup.fieldList) {
        if (item.lineList) {
          if (item.steerable) {
            // 汇总表格
            for (let d of item.lineList) {
              superGroupField = superGroupField.concat(d.fieldList)
            }
          } else {
            // 表格组
            // superGroupField = superGroupField.concat(item.lineList[0].fieldList)
          }
        } else {
          // 样式组
          if (item.groupType === GROUP_TYPE.super && item.fieldList) {
            superGroupField = superGroupField.concat(item.fieldList)
          } else {
            superGroupField.push(item)
          }
        }
      }
      if (rule.line && rule.line.fieldList) {
        superGroupField = superGroupField.concat(rule.line.fieldList)
      }
      let leftTF = []
      let rightTF = []
      if (conditionList.length !== 0) {
        for (let condition of conditionList) {
          if (condition.type === RULE_CONDITION_TYPE.formula) {
          // 公式
            let indexLeft = []
            let indexRight = []
            let indexLeftResult = ''
            let indexLeftResultReg = ''
            let indexRightResult = ''
            let indexRightResultReg = ''
            let index = condition.operatorList.findIndex(this.checkRelationalOperator)
            indexLeft = condition.operatorList.slice(0, index)
            indexRight = condition.operatorList.slice(index + 1)
            // index左边
            if (indexLeft.length === 1) {
              if (indexLeft[0].valueType === 2) {
              // 正则
                indexLeftResultReg = indexLeft[0]
              } else if (indexLeft[0].valueType === 1) {
                indexLeftResult = indexLeft[0].value
              } else {
                indexLeftResult = this.getFieldDefvalue(indexLeft[0], superGroupField)
              }
            } else {
              let NumberArr = []
              for (let leftD of indexLeft) {
                if (leftD.type === CALCULATE_TYPE.field) {
                  if (leftD.valueType === 1) {
                  // valueType为1，常量
                    NumberArr.push(leftD.value)
                  } else {
                    NumberArr.push(this.getFieldDefvalue(leftD, superGroupField))
                  }
                } else if (leftD.type === CALCULATE_TYPE.operator) {
                  NumberArr.push(this.getOperatorSwitch(leftD))
                }
              }
              NumberArr = formulaRule.reverseArray(NumberArr)
              indexLeftResult = formulaRule.executeFormula(NumberArr)
            }
            // index右边
            if (indexRight.length === 1) {
              if (indexRight[0].valueType === 2) {
              // 正则
                indexRightResultReg = indexRight[0].value
              } else if (indexRight[0].valueType === 1) {
                indexRightResult = indexRight[0].value
              } else {
                indexRightResult = this.getFieldDefvalue(indexRight[0], superGroupField)
              }
            } else {
              let NumberArr = []
              for (let leftD of indexRight) {
                if (leftD.type === CALCULATE_TYPE.field) {
                  if (leftD.valueType === 1) {
                  // valueType为1，常量
                    NumberArr.push(leftD.value)
                  } else {
                    NumberArr.push(this.getFieldDefvalue(leftD, superGroupField))
                  }
                } else if (leftD.type === CALCULATE_TYPE.operator) {
                  NumberArr.push(this.getOperatorSwitch(leftD))
                }
              }
              NumberArr = formulaRule.reverseArray(NumberArr)
              indexRightResult = formulaRule.executeFormula(NumberArr)
            }
            // return (||operator.operatorType ===  ||operator.operatorType ===  ||operator.operatorType ===  ||operator.operatorType === );

            switch (condition.operatorList[index].operatorType) {
              case OPERATOR_FUN_TYPE.equal:
              // 等于
                if (indexLeftResult === '') {
                  leftTF.push(indexLeftResultReg.test(String(indexRightResult)))
                }
                if (indexRightResult === '') {
                  leftTF.push(indexRightResultReg.test(String(indexLeftResult)))
                }
                if (indexLeftResult !== '' && indexRightResult !== '') {
                  leftTF.push(String(indexLeftResult) === String(indexRightResult))
                }
                break
              case OPERATOR_FUN_TYPE.greaterThan:
              // 大于
                leftTF.push(Number(indexLeftResult) > Number(indexRightResult))
                break
              case OPERATOR_FUN_TYPE.lessThan:
              // 小于
                leftTF.push(Number(indexLeftResult) < Number(indexRightResult))
                break
              case OPERATOR_FUN_TYPE.greaterThanAndEqual:
                leftTF.push(Number(indexLeftResult) >= Number(indexRightResult))
                // 大于等于
                break
              case OPERATOR_FUN_TYPE.lessThanAndEqual:
                leftTF.push(Number(indexLeftResult) <= Number(indexRightResult))
                // 小于等于
                break
              case OPERATOR_FUN_TYPE.notEqual:
              // 不等于
                if (indexLeftResult === '') {
                  leftTF.push(!new RegExp(indexLeftResultReg).test(String(indexRightResult)))
                }
                if (indexRightResult === '') {
                  leftTF.push(!new RegExp(indexRightResultReg).test(String(indexLeftResult)))
                }
                if (indexLeftResult !== '' && indexRightResult !== '') {
                  leftTF.push(String(indexLeftResult) !== String(indexRightResult))
                }
                break
            }
          } else if (condition.type === RULE_CONDITION_TYPE.logic) {
          // 逻辑运算符
            if (condition.logicOperator === OPERATOR_FUN_TYPE.or) {
              leftTF.push('or')
            } else if (condition.logicOperator === OPERATOR_FUN_TYPE.and) {
              leftTF.push('and')
            }
          }
        }
      }
      let leftresult
      if (leftTF.length === 0) {
        leftresult = true
      } else {
        while (leftTF.length > 0) {
          let operator = leftTF.pop()
          let nextValue
          if (operator === 'or') {
            nextValue = leftTF.pop()
            leftresult = leftresult || nextValue
          } else if (operator === 'and') {
            nextValue = leftTF.pop()
            leftresult = leftresult && nextValue
          } else {
            leftresult = operator
          }
        }
      }
      if (!leftresult) {
        callback()
      } else {
        for (let condition of resultList) {
          if (condition.type === RULE_CONDITION_TYPE.formula) {
            // 公式
            let indexLeft = []
            let indexRight = []
            let indexLeftResult = ''
            let indexLeftResultReg = ''
            let indexRightResult = ''
            let indexRightResultReg = ''
            let index = condition.operatorList.findIndex(this.checkRelationalOperator)
            indexLeft = condition.operatorList.slice(0, index)
            indexRight = condition.operatorList.slice(index + 1)
            // index左边
            if (indexLeft.length === 1) {
              if (indexLeft[0].valueType === 2) {
                // 正则
                indexLeftResultReg = indexLeft[0]
              } else if (indexLeft[0].valueType === 1) {
                indexLeftResult = indexLeft[0].value
              } else {
                indexLeftResult = this.getFieldDefvalue(indexLeft[0], superGroupField)
              }
            } else {
              let NumberArr = []
              for (let leftD of indexLeft) {
                if (leftD.type === CALCULATE_TYPE.field) {
                  if (leftD.valueType === 1) {
                    // valueType为1，常量
                    NumberArr.push(leftD.value)
                  } else {
                    NumberArr.push(this.getFieldDefvalue(leftD, superGroupField))
                  }
                } else if (leftD.type === CALCULATE_TYPE.operator) {
                  NumberArr.push(this.getOperatorSwitch(leftD))
                }
              }
              NumberArr = formulaRule.reverseArray(NumberArr)
              indexLeftResult = formulaRule.executeFormula(NumberArr)
            }
            // index右边
            if (indexRight.length === 1) {
              if (indexRight[0].valueType === 2) {
                // 正则
                indexRightResultReg = indexRight[0].value
              } else if (indexRight[0].valueType === 1) {
                indexRightResult = indexRight[0].value
              } else {
                indexRightResult = this.getFieldDefvalue(indexRight[0], superGroupField)
              }
            } else {
              let NumberArr = []
              for (let leftD of indexRight) {
                if (leftD.type === CALCULATE_TYPE.field) {
                  if (leftD.valueType === 1) {
                    // valueType为1，常量
                    NumberArr.push(leftD.value)
                  } else {
                    NumberArr.push(this.getFieldDefvalue(leftD, superGroupField))
                  }
                } else if (leftD.type === CALCULATE_TYPE.operator) {
                  NumberArr.push(this.getOperatorSwitch(leftD))
                }
              }
              NumberArr = formulaRule.reverseArray(NumberArr)
              indexRightResult = formulaRule.executeFormula(NumberArr)
            }
            switch (condition.operatorList[index].operatorType) {
              case OPERATOR_FUN_TYPE.equal:
                // 等于
                if (indexLeftResult === '') {
                  rightTF.push(new RegExp(String(indexLeftResultReg)).test(String(indexRightResult)))
                }
                if (indexRightResult === '') {
                  rightTF.push(new RegExp(String(indexRightResultReg)).test(String(indexLeftResult)))
                }
                if (indexLeftResult !== '' && indexRightResult !== '') {
                  const numL = Number(indexLeftResult)
                  const numR = Number(indexRightResult)
                  // 优先是number判定
                  if (!isNaN(numL) && !isNaN(numR)) {
                    rightTF.push(numL === numR)
                  } else {
                    rightTF.push(String(indexLeftResult) === String(indexRightResult))
                  }
                }
                break
              case OPERATOR_FUN_TYPE.greaterThan:
                // 大于
                rightTF.push(Number(indexLeftResult) > Number(indexRightResult))
                break
              case OPERATOR_FUN_TYPE.lessThan:
                // 小于
                rightTF.push(Number(indexLeftResult) < Number(indexRightResult))
                break
              case OPERATOR_FUN_TYPE.greaterThanAndEqual:
                rightTF.push(Number(indexLeftResult) >= Number(indexRightResult))
                // 大于等于
                break
              case OPERATOR_FUN_TYPE.lessThanAndEqual:
                rightTF.push(Number(indexLeftResult) <= Number(indexRightResult))
                // 小于等于
                break
              case OPERATOR_FUN_TYPE.notEqual:
                // 不等于
                if (indexLeftResult === '') {
                  rightTF.push(new RegExp(indexLeftResultReg).test(String(indexRightResult)))
                }
                if (indexRightResult === '') {
                  rightTF.push(new RegExp(indexRightResultReg).test(String(indexLeftResult)))
                }
                if (indexLeftResult !== '' && indexRightResult !== '') {
                  rightTF.push(String(indexLeftResult) === String(indexRightResult))
                }
                break
            }
          } else if (condition.type === RULE_CONDITION_TYPE.logic) {
            // 逻辑运算符
            if (condition.logicOperator === OPERATOR_FUN_TYPE.or) {
              rightTF.push('or')
            } else if (condition.logicOperator === OPERATOR_FUN_TYPE.and) {
              rightTF.push('and')
            }
          }
        }
        let rightresult
        while (rightTF.length > 0) {
          let operator = rightTF.pop()
          let nextValue
          if (operator === 'or') {
            nextValue = rightTF.pop()
            rightresult = rightresult || nextValue
          } else if (operator === 'and') {
            nextValue = rightTF.pop()
            rightresult = rightresult && nextValue
          } else {
            rightresult = operator
          }
        }
        if (!rightresult) {
          callback(new Error(rule.message))
        }
      }
      callback()
    }
  }
}
