/**
 * 检查是否是合法URL
 * @param {String} str 输入的URL
 * @returns {Boolean}
 */
export function validateURL (str) {
  const reg = /^(https?|ftp):\/\/([a-zA-Z0-9.-]+(:[a-zA-Z0-9.&%$-]+)*@)*((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]?[0-9])){3}|([a-zA-Z0-9-]+\.)*[a-zA-Z0-9-]+\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|pro|aero|coop|museum|[a-zA-Z]{2}))(:[0-9]+)*(\/($|[a-zA-Z0-9.,?'\\+&%$#=~_-]+))*$/
  return reg.test(str)
}

/**
 * 检查是否是小写字母
 * @param {String} str 输入的文本内容
 * @returns {Boolean}
 */
export function validateLowerCase (str) {
  const reg = /^[a-z]+$/
  return reg.test(str)
}

/**
 * 检查是否是大写字母
 * @param {String} str 输入的文本内容
 * @returns {Boolean}
 */
export function validateUpperCase (str) {
  const reg = /^[A-Z]+$/
  return reg.test(str)
}

/**
 *  检查是否是大小写字母
 * @param {String} str 输入的文本内容
 * @returns {Boolean}
 */
export function validateAlphabets (str) {
  const reg = /^[A-Za-z]+$/
  return reg.test(str)
}

/**
 * 检查是否是Email
 * @param {String} str 输入的文本内容
 * @returns {boolean}
 */
export function validateEmail (str) {
  /* eslint-disable */
  const re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
  return re.test(str)
}

/**
 * 检查密码: 8-20位字符，必须由数字、字母组成
 * @param {String} str 输入的密码
 * @returns {Boolean}
 */
export function validatePassword(str) {
  const reg = /^(?!([a-zA-Z]+|\d+)$)[a-zA-Z\d]{8,20}$/;
  return reg.test(str);
}

// 自定义验证函数，按照element rule标准编写
export default {
  /**
  * 数字范围验证
  */
  numberRange: function (min, max) {
    return (rule, value, callback) => {
      if (!value) {
        return callback()
      }
      value = Number(value)
      let tips
      if (min && max) {
        tips = '请输入范围 ' + min + ' 到 ' + max + ' 的数字'
      } else if (min) {
        tips = '请输入大于 ' + min + ' 的数字'
      } else if (max) {
        tips = '请输入小于 ' + max + ' 的数字'
      }
      if (value < min) {
        callback(new Error(tips))
      } else if (value > max) {
        callback(tips)
      } else {
        callback()
      }
    }
  },
  /**
   * 小数规则，如果decimal=false，在输入小数时直接去掉，不提示
   */
  decimalRule: function (decimal, decimalLength) {
    return (rule, value, callback) => {
      if (!value) {
        return callback()
      }
      if (!decimal) {
        // 是整数
        value = Number(value)
        Number.isInteger(value) ? callback() : callback(new Error('请输入整数'))
      } else if (decimal) {
        // 是小数
        if (value.toString().indexOf('.') !== -1) {
          if (value.toString().split('.')[1].length > decimalLength) {
            callback(new Error('请输入保留' + decimalLength + '位的小数'))
          } else {
            callback()
          }
        } else {
          callback()
        }
      }
    }
  },
  /**
   * 小数规则，如果decimal=false，在输入小数时直接去掉，不提示
   */
  decimalRule_prescription: function () {
    return (rule, value, callback) => {
      if (!value) {
        return callback(new Error('此项为必填项'))
      }
      if (/^[+]{0,1}(\d+)$|^[+]{0,1}(\d+\.\d+)$/.test(value)) {
        callback()
      } else {
        callback(new Error('此项为数字'))
      }
    }
  },
  /**
   * 中国身份证验证
   */
  chinaIdentityValid: (rule, value, callback) => {
    if (!value) {
      return callback()
    }
    let city = {
      11: '北京',
      12: '天津',
      13: '河北',
      14: '山西',
      15: '内蒙古',
      21: '辽宁',
      22: '吉林',
      23: '黑龙江 ',
      31: '上海',
      32: '江苏',
      33: '浙江',
      34: '安徽',
      35: '福建',
      36: '江西',
      37: '山东',
      41: '河南',
      42: '湖北 ',
      43: '湖南',
      44: '广东',
      45: '广西',
      46: '海南',
      50: '重庆',
      51: '四川',
      52: '贵州',
      53: '云南',
      54: '西藏 ',
      61: '陕西',
      62: '甘肃',
      63: '青海',
      64: '宁夏',
      65: '新疆',
      71: '台湾',
      81: '香港',
      82: '澳门',
      91: '国外'
    }
    if (!value || !/^\d{6}(18|19|20)?\d{2}(0[1-9]|1[012])(0[1-9]|[12]\d|3[01])\d{3}(\d|X)$/i.test(value)) {
      callback(new Error('身份证号格式错误'))
    } else if (!city[value.substr(0, 2)]) {
      callback(new Error('身份证号格式错误'))
    } else {
      // 18位身份证需要验证最后一位校验位
      if (value.length === 18) {
        const code = value.split('')
        // ∑(ai×Wi)(mod 11)
        // 加权因子
        const factor = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        // 校验位
        const parity = [1, 0, 'X', 9, 8, 7, 6, 5, 4, 3, 2]
        let sum = 0
        let ai = 0
        let wi = 0
        for (let i = 0; i < 17; i++) {
          ai = code[i]
          wi = factor[i]
          sum += ai * wi
        }
        if (parity[sum % 11].toString() !== code[17]) {
          callback(new Error())
        }
      }
    }
    callback()
  }
}
